# @o2xp/react-datatable

@o2xp/react-datatable is a modulable component to render data in a table with some features.

## Getting Started

### Installing 


```bash
$ npm i --save @o2xp/react-datatable
```

In your file

```jsx
// ES6
import {Datatable} from '@o2xp/react-datatable';
```

### Example

```jsx
let options  = {
  title: "My super datatable",
  dimensions: {
    datatable: {
      width: "90vw",
    },
    header: {
      height: "60px"
    },
    body: {
      height: "40vh"
    },
    row:{
      height: "60px"
    }
  },
  keyColumn: 'id',
  pagination:{
    rowsPerPage: {
      available: [10, 25, 50, 100],
      selected: 50
    }
  },
  userConfiguration: {
    columnsOrder: ["id", "name", "age"],
    copyToClipboard: true,
  },
  font: "Arial",
  dateFormat: "YYYY-MM-DDTHH:MM:ss",
  data: {
    columns: [ 
      {
        colSize: 10,
        label: "id",
        id: "id",
        disabled: true,
        dataType: "text"
      },
      {
        colSize: 20,
        label: "name",
        id: "name",
        disabled: false,
        dataType: "text"
      },
      {
        colSize: 3,
        label: "age",
        id: "age",
        disabled: true,
        dataType: "number"
      },
      {
        colSize: 0,
        label: "adult",
        id: "adult",
        inputType:"checkbox",
        disabled: false
      },
      {
        colSize: 10,
        label: "birth date",
        id: "birthDate",
        disabled: false,
        inputType: "date"
    },
    ],
    rows: [
      {
        id: "50cf",
        age: 28,
        name: "Kerr Mayo",
        adult: "Y",
        birthDate: "1972-09-04T11:09:59",
      },
      {
        id: "209",
        age: 34,
        name: "Freda Bowman",
        adult: "Y",
        birthDate: "1988-03-14T09:03:19"
      },
      {
        id: "2dd81ef",
        age: 14,
        name: "Becky Lawrence",
        adult: "N",
        birthDate: "1969-02-10T04:02:44"
      }
    ],
  },
  features: {
    editable: true,
    strippedRows: true,
    printable: true,
    downloadable: true,
    searchInput: true,
    canRefreshRows: true,
    canFilterColumns: true,
    canSaveColumnsConfiguration: true,
    selection: {
      rowsSelectable: true,
      selectPageRows: true,
      selectAllRows: false,
    },
    additionalIcons: [
      {
        tooltip: 'Coffee',
        onClick: () => this.coffeeTime('Coffee Time!'),
        icon: <CoffeeIcon/>
      },
      // DEPRECATED
      // {
        // tooltip: 'Warning Redux',
        // onClickDispatch: createNotification("warning", 'This is warning'),
        // icon: <WarningIcon/>
      // },
    ],
    selectionIcons: [
      {
        tooltip: 'Selected Rows',
        onClick: () => this.selectedRows(rows),
        icon: <CallSplitIcon />
      },
      // DEPRECATED
      // {
        // tooltip: 'Temporary Rate Assignment',
        // onClickDispatch: (selectedRows) => createNotification("error", [' + selectedRows.map(r => r.id) + ']'),
        // icon: <AssignIcon/>
      // }
    ]  
  }
}
```

In your render function :
```html
<Datatable data={options.data} options={options} refreshRows={this.refreshRows} noRedux={true} 
onChange={this.onChange} saveEditedRows={this.saveEditedRows} 
saveUserConfiguration={this.saveUserConfiguration}>
```

A complete basic running example can be found [here](https://github.com/o2xp/react-datatable-example).

To style a column content use this css class :
```sass
.o2 .Datatable-cell-<ID_OF_YOUR_COLUMN> {
  ...
}
```

To style stripped and checked rows use these css class :
```css
.o2 .Datatable-body-row-checked {
  ...
}

.o2 .Datatable-body-cell-stripped {
  ...
}
```


## Prop Types

| Property | Type | Required? | Description |
|:---|:---|:---:|:---|
| **title** | string | no | Title of the datatable. |
| **dimensions.datatable.width** | string | yes | Width of the of the Datatable. (in vw or px) |
| **dimensions.header.height** | string | yes | Height of the header of the Datatable. (in vh or px) |
| **dimensions.body.height** | string | yes | Height of the body of the Datatable. (in vh or px) |
| **dimensions.row.height** | string | yes | Height of the rows of the Datatable. (in vh or px) |
| **keyColumn** | string | no | Name of the column that has unique value and allows to identify a row. |
| **pagination.rowsPerPage.available** | array of numbers | yes | An array with the numbers of rows per page you want available. |
| **pagination.rowsPerPage.selected** | number | yes | The number of rows per page selected by default. |
| **userConfiguration.columnsOrder** | array of strings | no | An array of strings where the strings are the column identifier. Datatable will be rendered only with the columns present in the array. |
| **userConfiguration.copyToClipboard** | boolean | no | If true, when the user click on cell it will copy the value in the clipboard. |
| **font** | string | no | The name of the family font you are using, it allows us to calculate when a text doesn't fit a cell and needs a tooltip. (by default we use Arial) |
| **data.rows** | array of objects | yes | An array of objects where each object is defined by the columns identifier as key and the value. |
| **data.columns** | array of objects | yes | An array of objects where each object is defined by this keys : colsize (size of the columns in character number), label (label of the column), id (identifier of the column), inputType (value "checkbox" if it's a checkbox else ignore this key), dataType (value "number" if it's a number else ignore this key), disabled (if the field can be edit or no). Check the example just ahead |
| **feature.editable** | boolean | no | If the user can edit the rows. (Default: false) |
| **feature.editable** | boolean | no | If you want stripped rows. (Default: false) |
| **feature.printable** | boolean | no | If the user can print the datatable. (Default: false) |
| **feature.downloadable** | boolean | no | If the user can download the data. (Default: false) |
| **feature.searchInput** | boolean | no | If the user can filter the data by text through a search input. (Default: false) |
| **feature.canRefresh** | boolean | no | If the user can click to refresh the rows. (Default: false) |
| **feature.canFilterColumns** | boolean | no | If the user can select the columns to display. (Default: false) |
| **feature.canSaveColumnsConfifuration** | boolean | no | If the user can save his columns configuration. (order and which one is displayed) (Default: false) |
| **feature.selection.rowsSelectable** | boolean | no | If the user can select rows. (Default: false) |
| **feature.selection.selectPageRows** | boolean | no | If the user can do a global select and it selects all the rows of the current page. (Default: false) |
| **feature.selection.selectAllRows** | boolean | no | If the user can do a global select and it selects all the rows. (Default: false) |
| **feature.selection.additionalIcons** | array of objects | no | If you want to add icon which trigger a function or dispatch an redux action. (see example ahead) |
| **feature.selection.selectionIcons** | array of objects | no | If you want to add icon which dispatch an redux action or execute a function with the rows selected. (see example ahead) |
| **refreshRows** | function | no | Is triggered when the user click on the refresh button. |
| **onChange** | function | no | Send the rows when they are modified. |
| **saveEditedRows** | function | no | Send the rows that have been edited when the user clicks on the button. |
| **saveUserConfiguration** | function | no | Send the columns configuration of the user. |

## Built With

* [React](https://reactjs.org/) - A JavaScript library for building user interfaces
* [Redux](https://redux.js.org/) - Redux is a predictable state container for JavaScript apps
* [Material-ui](https://material-ui.com/) - React components that implement Google's Material Design
* [Webpack](https://github.com/webpack/webpack) - A bundler for javascript and friends. Packs many modules into a few bundled assets.
* [Babel](https://github.com/babel/babel) - Babel is a tool that helps you write code in the latest version of JavaScript. When your supported environments don't support certain features natively, Babel will help you compile those features down to a supported version.


## With the help of

* [fast-sort](https://github.com/snovakovic/fast-sort) - Blazing fast array sorting that outperforms lodash sorting by ~2x
* [json2csv](https://github.com/zemirco/json2csv) - Converts json into csv with column titles and proper line endings
* [material-ui-pickers](https://github.com/dmtrKovalenko/material-ui-pickers) - Accessible, customizable, delightful date & time pickers for @material-ui/core
* [moment](https://github.com/moment/moment/) - A lightweight JavaScript date library for parsing, validating, manipulating, and formatting dates.
* [react-easy-print](https://github.com/a-x-/react-easy-print) - Powerful react component for easy printing
* [react-number-format](https://github.com/s-yadav/react-number-format) - React component to format numbers in an input or as a text
* [react-search-input](https://github.com/enkidevs/react-search-input) - Simple React component for a search input, providing a filter function
* [react-sortable-hoc](https://github.com/clauderic/react-sortable-hoc) - A set of higher-order components to turn any list into an animated, touch-friendly, sortable list 
* [react-virtualized](https://github.com/bvaughn/react-virtualized) - React components for efficiently rendering large lists and tabular data
* [text-width](https://github.com/kapetan/text-width) - A small utility for measuring the text width in browsers that support the canvas API.
* [uuid](https://github.com/kelektiv/node-uuid) - Generate RFC-compliant UUIDs in JavaScript


## Authors

* [**Morgan Dubois**](https://github.com/MorganDbs) - *Initial work*
