import React, {Component, Fragment} from 'react';
import {IconButton, Tooltip, Input, Zoom, Checkbox, Select, MenuItem, Popper, DialogTitle, Grow, Paper, ClickAwayListener, MenuList, Dialog, Typography, DialogActions, Button, Switch, MuiDialogTitle} from '@material-ui/core';
import {Search as SearchIcon, Settings as SettingsIcon, Launch as LaunchIcon, Close as CloseIcon, Save as SaveIcon, History as HistoryIcon, Refresh as RefreshIcon, ViewColumn as ViewColumnIcon, CloudDownload as CloudDownloadIcon, Print as PrintIcon} from '@material-ui/icons';
import SearchInput, {createFilter} from 'react-search-input';
import {compareTwoArrays, deleteKeys, createArray} from '../DatatableContainerFunctions';
const Json2csvParser = require('json2csv').Parser;

class DatatableHeader extends Component {
  constructor(props) {
    super(props);
    this.state = {
      title: props.title,
      fileName: props.title.split(' ').join('_'),
      exportFileType: "csv",
      checkedRows: [],
      rows: [],
      searchTerm: '',
      filteredRows: [],
      keysToFilter: [],
      editedRows: [],
      columns: [],
      columnsToDisplay: [],
      rowsCurrentPage: [],
      additionalIcons: [],
      selectionIcons: [],
      dimensions: {},
      headerRef: {},
      bodyRef: {},
      canSave: false,
      viewRefreshOpen: false,
      viewColumnOpen: false,
      viewDownloadOpen: false,
      viewUserConfigurationOpen: false,
      changingIntern: false,
      activateRevertSave: false,
      editable: false,
      printable: false,
      downloadable: false,
      searchInput: false,
      canRefreshRows: false,
      canFilterColumns: false,
      canSaveColumnsConfiguration: false,
      searchInputOpen: false,
      rowSelectable: false,
      disabledIcon: false,
      keyColumn: '',
      userConfiguration: {
        columnsOrder: [],
        copyToClipboard: false,
      }
    };
  }
  
  static getDerivedStateFromProps(props, state) {
    let updatedState = {};
    let checkedRows = props.checkedRows;
    let rows = props.rows;
    let columns = props.columns;
    let columnsToDisplay = props.columnsToDisplay;
    let editedRows = props.editedRows;
    let userConfiguration = props.userConfiguration;
    let rowsCurrentPage = props.rowsCurrentPage;
    let additionalIcons = props.additionalIcons;
    let selectionIcons = props.selectionIcons;
    let editable = props.editable;
    let keyColumn = props.keyColumn;
    let printable = props.printable;
    let downloadable = props.downloadable;
    let searchInput = props.searchInput;
    let canRefreshRows = props.canRefreshRows;
    let canFilterColumns = props.canFilterColumns;
    let canSaveColumnsConfiguration = props.canSaveColumnsConfiguration;
    let rowSelectable = props.rowSelectable;
    let headerRef = props.headerRef;
    let bodyRef = props.bodyRef;
    let dimensions = props.dimensions;
    let searchTerm = state.searchTerm;
    let changingIntern = state.changingIntern;
    
    if(!compareTwoArrays(checkedRows, state.checkedRows)){
      updatedState.checkedRows = checkedRows;
    }
    
    if(!compareTwoArrays(editedRows, state.editedRows)){
      updatedState.editedRows = editedRows;
    }
    
    if(!compareTwoArrays(rowsCurrentPage, state.rowsCurrentPage)){
      updatedState.rowsCurrentPage = rowsCurrentPage;
    }
    
    updatedState.keysToFilter = columnsToDisplay.map(column => column.id);
    
    if(!compareTwoArrays(columns, state.columns)){
      updatedState.columns = columns;
    }
    
    if(!compareTwoArrays(columnsToDisplay, state.columnsToDisplay)){
      updatedState.columnsToDisplay = columnsToDisplay;
    }
    
    if(!compareTwoArrays(rows, state.rows)){
      if(searchTerm !== ''){
        props.getFilteredRows(rows.filter(createFilter(searchTerm, updatedState.keysToFilter)), searchTerm);
      }
      updatedState.rows = rows;
    }
    
    if(!changingIntern && JSON.stringify(userConfiguration) !== JSON.stringify(state.userConfiguration)){
      updatedState.userConfiguration = userConfiguration;
    }else if(changingIntern){
      updatedState.changingIntern = false;
    }
    
    
    if(JSON.stringify(dimensions) !== JSON.stringify(state.dimensions)){
      updatedState.dimensions = dimensions;
    }
    
    if(userConfiguration.columnsOrder.length > 0){
      updatedState.canSave = !compareTwoArrays(userConfiguration.columnsOrder, columnsToDisplay.map(col => col.id));
    }else{
      updatedState.canSave = !compareTwoArrays(columnsToDisplay.map(col => col.id), columns.map(col => col.id));
    }
    
    if(headerRef !== state.headerRef){
      updatedState.headerRef = headerRef;
    }
    
    if(bodyRef !== state.bodyRef){
      updatedState.bodyRef = bodyRef;
    }
    
    if(additionalIcons.length > 0){
      updatedState.additionalIcons = additionalIcons;
    }
    
    if(selectionIcons.length > 0){
      updatedState.selectionIcons = selectionIcons;
    }

    if(editable !== state.editable){
      updatedState.editable = editable;
    }

    if(keyColumn !== state.keyColumn){
      updatedState.keyColumn = keyColumn;
    }
    
    if(printable !== state.printable){
      updatedState.printable = printable;
    }
    
    if(downloadable !== state.downloadable){
      updatedState.downloadable = downloadable;
    }
    
    if(searchInput !== state.searchInput){
      updatedState.searchInput = searchInput;
    }
    
    if(canRefreshRows !== state.canRefreshRows){
      updatedState.canRefreshRows = canRefreshRows;
    }
    
    if(canFilterColumns !== state.canFilterColumns){
      updatedState.canFilterColumns = canFilterColumns;
    }
    
    if(canSaveColumnsConfiguration !== state.canSaveColumnsConfiguration){
      updatedState.canSaveColumnsConfiguration = canSaveColumnsConfiguration;
    }
    
    if(rowSelectable !== state.rowSelectable){
      updatedState.rowSelectable = rowSelectable;
    }
    
    if(columns.length === 0 || rows.length === 0){
      updatedState.disabledIcon = true;
    }
    
    return updatedState;
  }
  
  activateRevertSave = (bool) => {
    this.setState({activateRevertSave: bool})
  }
  
  /* Return the rows that have been checked */
  getCheckedRows = (icon) => {
    let checkedRows = this.state.checkedRows;
    let keyColumn = this.state.keyColumn;
    let rowsRefactored = []
    
    checkedRows.forEach(row => {
      let r = {};
      Object.keys(row).forEach(key => {
        if(key !== "o2DatatableRowUuid" && key !== "o2DatatableRowSelected"){
          r[key] = row[key];
        }
      });
      rowsRefactored.push(r);
    });
    
    if(icon.onClick) {
      icon.onClick(rowsRefactored)
    }
    
    if(icon.onClickDispatch){
      this.props.handleSelectionIconDispatch(icon.onClickDispatch, rowsRefactored)
    } 
    
    this.props.resetRowsSelected();
  }
  
  additionalIconsClick = (icon) => {
    if(icon.onClick) {
      icon.onClick()
    }
    
    if(icon.onClickDispatch){
      this.props.handleAdditionalIconDispatch(icon.onClickDispatch)
    } 
  }
  
  
  /* Update the rows by the searchTerm */
  searchUpdated = (term) => {
    let regExp = new RegExp('^(\\d+,)+(\\d+)?(\\.\\d+)?$');
    if(regExp.test(term)){
      term = term.split(',').join('');
    }
    let rows = this.state.rows;
    let keysToFilter = this.state.keysToFilter;
    let filteredRows = term.length > 0 ? rows.filter(createFilter(term, keysToFilter)) : [];
    
    this.props.setPage(1);
    this.setState({searchTerm: term});
    this.setState({filteredRows: filteredRows});    
    this.props.getFilteredRows(filteredRows, term);
  }
  
  /* Return edited rows */
  saveEditedRows = () => {
    this.props.saveEditedRows();
  }
  
  /* Reseting the array of edited rows */
  resetEditedRows = () => {
    this.props.resetEditedRows();
  }
  
  /* Refreshing the rows */
  refreshRows = () => {
    this.props.refreshRows();
  }
  
  /* Toggle the download modal */
  toggleViewDownload = (e) => {
    this.setState({
      viewDownloadOpen: !this.state.viewDownloadOpen
    });
  }
  
  /* Toggle the column visibility modal */
  toggleViewColumn = (e) => {
    this.setState({
      viewColumnOpen: !this.state.viewColumnOpen
    });
  }
  
  /* Close the column visibility modal */
  closeViewColumn = (e) => {
    if (this.anchorViewColumn.contains(e.target)) {
      return;
    }
    this.setState({viewColumnOpen: false});
  }
  
  /* Close the download modal */
  closeViewDownload = () => {
    this.setState({viewDownloadOpen: false});
  }
  
  /* Download the data in a csv file */
  downloadCsv = (type) => {
    let checkedRows = this.state.checkedRows;
    let rowsCurrentPage = this.state.rowsCurrentPage;
    let rows = this.state.rows;
    let exportFileType = this.state.exportFileType;
    let fileName = this.state.fileName;
    let columns = createArray(this.state.columns);
    let data = {};
    
    switch (type) {
      case "checkedRows":
        data = checkedRows;
        break;
      case "rowsCurrentPage":
        data = rowsCurrentPage;
        break;
      case "rows":
        data = rows;
        break;    
      default:
        break;
    }
    
    data = deleteKeys(createArray(data), ["o2DatatableRowUuid", "o2DatatableRowSelected"]);
    let hiddenElement = document.createElement('a');
          
    if(exportFileType === "csv"){
      let json2csvParser = new Json2csvParser({ columns });
      let csv = json2csvParser.parse(data);    
      hiddenElement.href = 'data:text/csv;charset=utf-8,' + encodeURI(csv);
      hiddenElement.target = '_blank';
      hiddenElement.download = fileName + '.csv';      
    }else{
      hiddenElement.href = 'data:text/json;charset=utf-8,' + encodeURIComponent(JSON.stringify(data));
      hiddenElement.target = '_blank';
      hiddenElement.download = fileName + '.json';  
    }
    
    hiddenElement.click();
    this.closeViewDownload();
  }
  
  /* Change the visibility of a column */
  toggleColumnVisiblity = (id) => {
    let columns = this.state.columns;
    columns.forEach(col => {
      if(col.id === id){
        col.o2DatatableColumnVisible = !col.o2DatatableColumnVisible;
      }
    });
    
    this.state.bodyRef.recomputeGridSize();
    this.state.headerRef.recomputeGridSize();
    this.props.setColumnsVisibility(columns);
  }
  
  /* Refresh the data of the datatable */
  refreshRows = () => {
    let editedRows = this.state.editedRows;
    let checkedRows = this.state.checkedRows;
    if(editedRows.length > 0 || checkedRows.length > 0){
      this.setState({viewRefreshOpen: true});
    }else{
      this.props.refreshRows();
    }
  }
  
  /* Closing the modal of warning when the user want to refresh */
  closeModalRefreshRows = (refreshing) => {
    this.setState({viewRefreshOpen: false});
    if(refreshing){
      this.props.refreshRows();
    }
  }
  
  /* Print the datatable */
  printDatatable = () => {
    setTimeout(() => window.print(), 500);
  }
  
  /* Change the type of the exported file */
  changeExportFileType = (el) => {
    this.setState({exportFileType: el.target.value});
  }
  
  /* Change the name of the file */
  changeFileName = (el) => {
    this.setState({fileName: el.target.value});
  }
  
  /* Select all the text of the input */
  handleFocus = (event) => {
    event.target.select();
  }
  
  saveColumnsConfiguration = () => {
    this.closeViewUserConfiguration(true);
  }
  
  resetColumnsConfiguration = () => {
    this.closeViewUserConfiguration(false);
  }
  
  /* Close the modal of user configuration */
  closeViewUserConfiguration = (save) => {
    this.setState({viewUserConfigurationOpen: false});
    let different = this.state.userConfiguration.copyToClipboard !== this.props.userConfiguration.copyToClipboard ? true : false;
    let copyToClipboard = this.state.userConfiguration.copyToClipboard;
    save = different ? true : save;
    switch (save) {
      case true: {
        this.props.saveUserConfiguration(this.state.userConfiguration);
        break;
      }
      case false: {
        this.props.resetUserConfiguration(different ? copyToClipboard : null);
        break;
      }
    }
  }
  
  /* Reset the configuration */
  resetConfiguration = () => {
    this.closeViewUserConfiguration();
    this.props.resetUserConfiguration();
  }
  
  
  /* Open the modal of user configuration */
  openViewUserConfiguration = () => {
    this.setState({viewUserConfigurationOpen: true});
  }
  
  /* Toggle search Input */
  toggleSearchInput = () => {
    this.setState({searchInputOpen: !this.state.searchInputOpen});
  }

  render() {
    const { disabledIcon, columns, keyColumn, activateRevertSave, searchInputOpen, rowSelectable, dimensions, selectionIcons, 
            canSaveColumnsConfiguration, editable, printable, downloadable, searchInput, canRefreshRows, canFilterColumns, checkedRows, title, 
            fileName, canSave, additionalIcons, exportFileType, editedRows, rowsCurrentPage, viewUserConfigurationOpen, viewColumnOpen, 
            viewDownloadOpen, viewRefreshOpen, userConfiguration
          } = this.state;
    return (  
      <div className={'Datatable-header'} style={{height: dimensions.header.height}}>
        <div className={"Datatable-header-title"}>              
          {title}
        </div>
        
        {
          searchInput && !searchInputOpen &&
          <Grow in={!searchInputOpen} timeout={300}>
            <div>
              <Tooltip TransitionComponent={Zoom} title="Search">
                <span>
                  <IconButton aria-label="Search" disabled={disabledIcon} className={!disabledIcon ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={() => this.toggleSearchInput()}>
                    <SearchIcon />
                  </IconButton>          
                </span>
              </Tooltip>
            </div>
          </Grow>
        }
        
        {
          searchInput && searchInputOpen &&
          <Grow in={searchInputOpen} timeout={300} >
            <div style={{display: "flex", alignItems: "center"}}>
              <Tooltip TransitionComponent={Zoom} title="Search">
                <span>
                  <IconButton aria-label="Search" color="primary" onClick={() => this.toggleSearchInput()}>
                    <SearchIcon />
                  </IconButton>          
                </span>
              </Tooltip> 
              <SearchInput className="Datatable-search-input" throttle={10} onChange={this.searchUpdated} autoFocus/>
            </div>
          </Grow>
        }
        
        {
          editable &&
          <div>
            <Tooltip TransitionComponent={Zoom} title="Revert changes">
              <span>
                <IconButton aria-label="Reset" disabled={disabledIcon || !activateRevertSave} className={activateRevertSave ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={() => this.resetEditedRows()}>
                  <HistoryIcon />
                </IconButton>          
              </span>
            </Tooltip>  
          </div>        
        }
        
        {
          editable &&
          <div>
            <Tooltip TransitionComponent={Zoom} title="Save changes">
              <span>
                <IconButton aria-label="Save" disabled={disabledIcon || !activateRevertSave} className={activateRevertSave ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={() => this.saveEditedRows()}>
                  <SaveIcon />
                </IconButton>
              </span>
            </Tooltip>     
          </div>     
        }
        
        {rowSelectable && 
          selectionIcons.map((icon, i) => {
            return (
              <div key={i}>
                <Tooltip TransitionComponent={Zoom} title={icon.tooltip}>
                  <span>
                    <IconButton 
                      disabled={checkedRows.length === 0 ? true : false || disabledIcon}
                      aria-label={icon.tooltip} 
                      aria-haspopup="true" 
                      className={checkedRows.length === 0 || disabledIcon ? "" : "Datatable-header-icons-selection"} 
                      onClick={() => this.getCheckedRows(icon)}
                    >
                      {icon.icon}
                    </IconButton>
                  </span>
                </Tooltip>
              </div>
            )
          })          
        } 
        
        {
          canRefreshRows &&
          <div>
            <Tooltip TransitionComponent={Zoom} title="Refresh data">
              <span>
                <IconButton aria-label="Refresh" className={"Datatable-header-icons"} onClick={() => this.refreshRows()}>
                  <RefreshIcon />
                </IconButton>
              </span>
            </Tooltip>          
          </div>
        }
        
        {
          canRefreshRows &&
          <Dialog
            open={viewRefreshOpen}
            aria-labelledby="alert-dialog-title"
            aria-describedby="alert-dialog-description"
          >
            <Typography variant="h6" className={"Datatable-modalTitle"}>
              Do you really want to refresh the data ?
              <IconButton className={"Datatable-header-dialog-close"} aria-label="Close" onClick={() => this.closeModalRefreshRows(false)}>
                <CloseIcon />
              </IconButton>
            </Typography>
              
            <div className={"Datatable-modalContent"}>
              <Typography variant="subtitle1">
                { editedRows.length > 0 && "You have unsaved modifications, if you refresh the data you will lost your modifications."}
                { checkedRows.length > 0 && "You have selected rows, if you refresh the data we will unselect all the rows."}
                <br/>
                Do you want to continue ?
              </Typography>
            </div>
              
            <DialogActions>
              <Button size="small" variant="outlined" onClick={() => this.closeModalRefreshRows(false)} color="secondary">
                Cancel
              </Button>
              <Button size="small" variant="outlined" onClick={() => this.closeModalRefreshRows(true)} className={"Datatable-header-icons"}>
                Continue
              </Button>
            </DialogActions>
          </Dialog>       
        }
        
        {
          canFilterColumns &&
          <div>
            <Tooltip TransitionComponent={Zoom} title={viewColumnOpen ? "" : "View column"}>
              <span>
                <IconButton buttonRef={node => {this.anchorViewColumn = node;}} disabled={disabledIcon} aria-label="ViewColumn" aria-haspopup="true" className={!disabledIcon ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={(e) => this.toggleViewColumn(e)}>
                  <ViewColumnIcon />
                </IconButton>
              </span>
            </Tooltip>
          </div>  
        }    
        
        {
          canFilterColumns &&
          <Popper open={viewColumnOpen} anchorEl={this.anchorViewColumn} style={{zIndex: 9999999}} transition disablePortal>
            {({ TransitionProps, placement }) => (
              <Grow
                {...TransitionProps}
                id="menu-list-grow"
                style={{ transformOrigin: 'center bottom'}}
              >
                <Paper style={{ maxHeight: 400, maxWidth: 250, overflow: 'auto'}}>
                  <ClickAwayListener onClickAway={(e) => this.closeViewColumn(e)}>
                    <MenuList>
                      {columns.map(column => {
                        if(column.id !== "selectable"){
                          return (
                            <MenuItem key={column.id} onClick={() => this.toggleColumnVisiblity(column.id)}>
                              <Checkbox
                                checked={column.o2DatatableColumnVisible}
                                className={"Datatable-header-icons"}
                              />
                              {column.label}
                            </MenuItem>
                          )
                        }
                        return null;
                      }).filter(el => el !== null)}
                    </MenuList>
                  </ClickAwayListener>
                </Paper>
              </Grow>
            )}
          </Popper>    
        }    
        
        {
          downloadable &&
          <div>
            <Tooltip TransitionComponent={Zoom} title={viewDownloadOpen ? "" : "Download data"}>
              <span>
                <IconButton aria-label="Download" disabled={disabledIcon} aria-haspopup="true" className={!disabledIcon ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={() => this.toggleViewDownload()}>
                  <CloudDownloadIcon />
                </IconButton>
              </span>
            </Tooltip>
          </div>
        }
        
        {
          downloadable &&
          <Dialog
            open={viewDownloadOpen}
            onClose={this.closeViewDownload}
            aria-labelledby="alert-dialog-title"
            aria-describedby="alert-dialog-description"
          >
            <Typography variant="h6" className={"Datatable-modalTitle"}>
              Download Data
              <IconButton className={"Datatable-header-dialog-close"} aria-label="Close" onClick={this.closeViewDownload}>
                <CloseIcon />
              </IconButton>
            </Typography>
              
            {
              editedRows &&
              <div className={"Datatable-modalContent"}>
                <Typography variant="subtitle1">
                  The data will be exported in a {exportFileType} file.
                </Typography>
                <br/>
                <Input
                  id="outlined-name"
                  label={fileName}
                  value={fileName}
                  error={fileName.split(' ').join('').length > 0 ? false : true}
                  onChange={this.changeFileName}
                  onFocus={this.handleFocus}
                  autoFocus
                />
                <Select
                  position="end"
                  value={exportFileType}
                  onChange={this.changeExportFileType}
                >
                  <MenuItem value={"csv"}>.csv</MenuItem>
                  <MenuItem value={"json"}>.json</MenuItem>
                </Select>
              </div>
            }
                
            {
              editedRows.length > 0 &&
              <div className={"Datatable-modalContent"}>
                <Typography variant="subtitle1">
                  It appears that you have unsaved change. Reset it or save before downloading data.
                </Typography>
                
                <Tooltip TransitionComponent={Zoom} title="Revert changes">
                  <span>
                    <IconButton aria-label="Reset" disabled={disabledIcon} color="secondary" onClick={() => this.resetEditedRows()}>
                      <HistoryIcon />
                    </IconButton>          
                  </span>
                </Tooltip>
                    
                <Tooltip TransitionComponent={Zoom} title="Save changes">
                  <span>
                    <IconButton aria-label="Save" disabled={disabledIcon} className={"Datatable-header-icons"} onClick={() => this.saveEditedRows()}>
                      <SaveIcon />
                    </IconButton>
                  </span>
                </Tooltip>              
              </div>
            }
                
            <DialogActions>
              <Button size="small" variant="outlined" disabled={checkedRows.length === 0 || editedRows.length > 0 || disabledIcon || fileName.split(' ').join('').length === 0} onClick={() => this.downloadCsv("checkedRows")}>
                Rows Selected
              </Button>
              <Button size="small" variant="outlined" disabled={editedRows.length > 0 || rowsCurrentPage.length === 0  || disabledIcon || fileName.split(' ').join('').length === 0} onClick={() => this.downloadCsv("rowsCurrentPage")}>
                Rows of current page
              </Button>
              <Button size="small" variant="outlined" disabled={editedRows.length > 0  || disabledIcon || fileName.split(' ').join('').length === 0} onClick={() => this.downloadCsv("rows")}>
                All rows
              </Button>
            </DialogActions>
          </Dialog>
        }
        
        {
          printable &&
          <div>
            <Tooltip TransitionComponent={Zoom} title="Print datatable">
              <span>
                <IconButton aria-label="PrintDatatable" aria-haspopup="true" disabled={disabledIcon} className={!disabledIcon ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={() => this.printDatatable()}>
                  <PrintIcon />
                </IconButton>
              </span>
            </Tooltip>
          </div>
        }    
        
        <div>
          <Tooltip TransitionComponent={Zoom} title="Save configuration">
            <span>
              <IconButton disabled={disabledIcon} aria-label="SaveConfiguration" aria-haspopup="true" className={!disabledIcon ? "Datatable-header-icons" : "Datatable-header-icons-disabled"} onClick={() => this.openViewUserConfiguration()}>
                <SettingsIcon />
              </IconButton>
            </span>
          </Tooltip>
        </div>
        
        <Dialog
          open={viewUserConfigurationOpen}
          onClose={this.closeViewUserConfiguration}
          aria-labelledby="alert-dialog-title"
          aria-describedby="alert-dialog-description"
        >
          <Typography variant="h6" className={"Datatable-modalTitle"}>
            User configuration
            <IconButton className={"Datatable-header-dialog-close"} aria-label="Close" onClick={this.closeViewUserConfiguration}>
              <CloseIcon />
            </IconButton>
          </Typography>
          <div className={"Datatable-modalContent"}>
            <Typography variant="subtitle1">
              Save cell's content to clipboard on click : 
              <Switch 
                checked={userConfiguration.copyToClipboard}
                onChange={() => this.setState({
                  userConfiguration: {
                    columnsOrder: userConfiguration.columnsOrder,
                    copyToClipboard: !userConfiguration.copyToClipboard
                  },
                  changingIntern: true,
                })}
                color="primary"
              />
            </Typography>
              
            {
              canSaveColumnsConfiguration &&
              <Fragment>
                <hr/>
                <Typography variant="subtitle1" style={{marginTop: "10px", marginBottom: "20px"}}>
                  Do you want to save the configuration of the columns of the datatable ?
                </Typography>      
                  
                <DialogActions>
                  <Button size="small" variant="outlined" disabled={!canSave} onClick={() => this.resetColumnsConfiguration()} color="secondary">
                    Reset
                  </Button>
                  <Button size="small" variant="outlined" disabled={!canSave} onClick={() => this.saveColumnsConfiguration()} className={"Datatable-header-icons"}>
                    Save
                  </Button>
                </DialogActions>
              </Fragment>
            }
          </div>
        </Dialog>
        {
          additionalIcons.map((icon, i) => {
            return (
              <div key={i}>
                <Tooltip TransitionComponent={Zoom} title={icon.tooltip}>
                  <span>
                    <IconButton 
                      aria-label={icon.tooltip} 
                      disabled={disabledIcon}
                      aria-haspopup="true" 
                      className={!disabledIcon ? "Datatable-header-icons" : "Datatable-header-icons-disabled"}
                      onClick={() => this.additionalIconsClick(icon)}
                    >
                      {icon.icon}
                    </IconButton>
                  </span>
                </Tooltip>
              </div>
            )
          })          
        }      
      </div>
    );
  }
}

export {DatatableHeader};